/*!
 ***********************************************************************
 *  @file	     adc.h
 *  @par Project   : c't Mikrocontroller-im-LAN --> Wetterstation
 *  @par Module    : ADC-Wandler
 *
 *  @brief	Funktionen zum einlesen analoger Werte
 *
 *  @author Benjamin Benz (bbe@heise.de)
 *  @author Markus Thiele (Chewy.F5@gmx.de)
 *  @version 2.0
 *	@date 08.01.2005
 ***********************************************************************
 *
 *  @par History   :
 *  @verbatim
 *  Version | Date       | Author    	      | Change Description
 *  1.0     | 24.06.2004 | Benz               | First Version, Demo Source Code
 *  2.0     | 08.01.2005 | Thiele             | First Documentated Version of Wetterstation
 *  @endverbatim
 *
 ************************************************************************/
/*!
 **********************************************************************
 * @fn int read_adc(unsigned char channel)
 *
 * @brief Liest einen Kanal ein
 *
 * \todo Sleepmodus einbauen so das es nicht die Funktion send_all stoert
 *
 * @param  channel   - IN, Kanalnummer
 * \n                  0 = PT1
 * \n                  1 = PT2
 * \n                  2 = LDR
 * \n                  3 = Druck
 * \n                  4 = Feuchte
 *
 * @retval           - Wert des eingelesenen Kanals
 ************************************************************************/
/*!
 **********************************************************************
 * @fn short read_zahl_adc(unsigned char channel)
 *
 * @brief Liest einen Kanal ein und formatiert sie in die richtige Einheit
 *
 * @param  channel   - IN, Kanalnummer
 * \n                  0 = PT1
 * \n                  1 = PT2
 * \n                  2 = LDR
 * \n                  3 = Druck
 * \n                  4 = Feuchte
 *
 * @retval           - Wert des eingelesenen Kanals korrigiert die Messeinheit
 ************************************************************************/
/*!
 **********************************************************************
 * @fn void read_string_adc(unsigned char channel,unsigned char *data)
 *
 * @brief Liest einen Kanal ein
 *
 * @param  channel   - IN, Kanalnummer
 * \n                  0 = PT1
 * \n                  1 = PT2
 * \n                  2 = LDR
 * \n                  3 = Druck
 * \n                  4 = Feuchte
 * @param  data      - Out, String mit Wert+Einheit
 ************************************************************************/

//Liest einen analogen kanal aus
//Der Parameter ist die hex-Wertigkeit des Pins (0x01 fr PA0; 0x02 fr PA1, ..)
int read_adc(unsigned char channel);
short read_zahl_adc(unsigned char channel);
void read_string_adc(unsigned char channel,unsigned char *data);
