/* c't Mikrocontroller-im-LAN Demo Source
 * Datei: adc.c
 * Info: Routinen zum Einlesen der Analogeingnge
 * Autor: Benjamin Benz (bbe@heise.de)
 * Datum: 24.06.04
 *
 * 26. Jan. 2005 wb
 *      Interrupt ADC-Complete
 *      ToDo: Sttzstellen im eeProm
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>

#include <delay.h>
#include <math.h>
#include "adc.h"
#include "led.h"
#include "tools.h"
#include "display.h"
#include "mcu.h"

// Zum Kalibrieren bentigen wir zwei Sttzstellen. Den Rest macht der Precompiler fr uns :-)
// Bitte entsprechend eintragen


//Sensor 0
#define pt100_0_messwert_1 485.00//540.00       // Messwert 2
#define pt100_0_realwert_1 16.40 //20.50        // passende temperatur in hunderstel Grad

#define pt100_0_messwert_2 578.00 //260.00       // Messwert 1
#define pt100_0_realwert_2 20.70 //4.30         // passende temperatur in hunderstel Grad

//Sensor 1
#define pt100_1_messwert_1 512 //543.00       // Messwert 2
#define pt100_1_realwert_1 16.4 //20.50       // passende temperatur in hunderstel Grad

#define pt100_1_messwert_2 571.0 //295.00       // Messwert 1
#define pt100_1_realwert_2 20.70 //4.00         // passende temperatur in hunderstel Grad


// Nicht bearbeiten!!!
#define pt100_0_faktor ((pt100_0_realwert_1-pt100_0_realwert_2)/(pt100_0_messwert_1-pt100_0_messwert_2))
#define pt100_0_offset (pt100_0_realwert_1- (pt100_0_faktor*pt100_0_messwert_1))
#define pt100_1_faktor ((pt100_1_realwert_1-pt100_1_realwert_2)/(pt100_1_messwert_1-pt100_1_messwert_2))
#define pt100_1_offset (pt100_1_realwert_1- (pt100_1_faktor*pt100_1_messwert_1))

const float korr_faktor[] = { pt100_0_faktor, pt100_1_faktor };
const float korr_offset[] = { pt100_0_offset, pt100_1_offset };


 
static char lastChannel = 0;
static int  adcValues[N_ADC_CHANNELS] = {0,0,0};

/*
 * Liest einen analogen kanal aus
 * Der Parameter ist die Nummer des Kanals (0 fr PA0, 1 fr PA1, ..)
 */
int read_adc(char channel)
{
        int result;
        if(channel >= N_ADC_CHANNELS)
            return -1; 
        char sreg = SREG;
        cli();
        result = adcValues[channel];
        SREG = sreg;
        return result;
}

/*
 * Liefert die Temperatur der PT100 Sensoren in Hunderstel Grad Celsius zurck
 */
int pt100_read(char channel)
{
//        return read_adc(channel);
        float temp = (float)read_adc(channel);   // Wert auslesen
        temp = ((korr_faktor[(int)channel] * temp) + korr_offset[(int)channel]);
        return floor(temp*100);
}
 
/*
 * Initialisieren der AD-Wandlung und Neustart nach stopADC()
 */
void init_adc(void)
{
    
        DDRA  &= ~7;    // 3 Pins als input
        PORTA &= ~7;    // Alle Pullups der drei pins aus.

        ADMUX = (1<<REFS1) | (1<<REFS0) | (0<<ADLAR); // interne Refernzspannung, rechts Ausrichtung

        ADMUX = (1<<REFS1) | (1<<REFS0) | (0<<ADLAR) | lastChannel;
       
//        ADCSRA = (1<<ADPS2) + (1<<ADPS1)+        // prescale faktor = 64 ADC luft mit 14,7456MHz / 64 = 230,4 kHz
        ADCSRA = (1<<ADPS2) + (1<<ADPS1) + (1<<ADPS0) +        // prescale faktor = 128 ADC luft mit 14,7456MHz / 128 = 150,2 kHz
                 (1<<ADEN)  +                    // ADC an
                 (1<<ADSC)  + (1<<ADIE);         // Beginne mit der Konvertierung, feuere Interrupt wenn efrtig
}

/*
 *  Manchmal kann es sinnvoll sein das Teil zu stoppen...z.B. fr SLEEP-Mode
 */ 
void stopADC()
{
    ADCSRA = 0;                         // Deaktiviere ADC nach aktueller konvertierung
    while ( (ADCSRA & (1<<ADSC)) != 0)  // Warten bis konvertierung beendet
        wdt_reset();                    // Sonst laft ADC-Power im Sleep weiter
    
}


/*
 * AD Conversion Complete Interrupt
 */
SIGNAL (SIG_ADC)
{
    adcValues[lastChannel] = ADCL | (ADCH << 8);      // Ergebnis zusammenbauen
    lastChannel++;
    if(lastChannel >= N_ADC_CHANNELS)
        lastChannel = 0;
    // Nchsten kanal
    ADMUX = (1<<REFS1) | (1<<REFS0) | (0<<ADLAR) | lastChannel;

//    ADCSRA = (1<<ADPS2) + (1<<ADPS1)+        // prescale faktor = 64 ADC luft mit 14,7456MHz / 64 = 230,4 kHz
    ADCSRA = (1<<ADPS2) + (1<<ADPS1) + (1<<ADPS0) +        // prescale faktor = 128 ADC luft mit 14,7456MHz / 128 = 150,2 kHz
             (1<<ADEN)  +                    // ADC an
             (1<<ADSC)  + (1<<ADIE);         // Beginne mit der Konvertierung, feuere Interrupt wenn fertig
}
 